-- @(#) md5agg(text) aggregate function for PostgreSQL in pure SQL
-- $Header: /code/pg-mdagg/md5aggslow.sql,v 1.1.1.1 2009/07/08 07:29:49 doj Exp $

-- Warning! These functions may consume large amounts of server
-- memory, depending on the combined text size of columns fed into the
-- MD5 aggregate. Use the C function implementation md5agg.sql if you
-- can install C function on your PostgreSQL installation.

CREATE OR REPLACE FUNCTION md5agg_state(text, text)
RETURNS text
AS 'SELECT $1 || $2;'
LANGUAGE SQL STRICT;
COMMENT ON FUNCTION md5agg_state(text, text) IS 'State function for md5agg(text) aggregate.';

CREATE OR REPLACE FUNCTION md5agg_final(text)
RETURNS text
AS 'SELECT md5($1);'
LANGUAGE SQL STRICT;
COMMENT ON FUNCTION md5agg_final(text) IS 'Finalize function for md5agg(text) aggregate.';

DROP AGGREGATE IF EXISTS md5agg(text);
CREATE AGGREGATE md5agg(text) (
  SFUNC = md5agg_state,
  STYPE = text,
  FINALFUNC = md5agg_final
);
COMMENT ON AGGREGATE md5agg(text) IS 'MD5 aggregate function implemented in pure SQL code. This aggregate function will construct an in-memory temporary variable containing all concatenated column texts, so usage of this function may allocate large amounts of memory. You have been warned! If you can install C functions for PostgreSQL, use md5agg.sql to install the aggregate function.';
